<?php

namespace Ems\Controllers;

use Ems\Services\MarcacaoService;
use Ems\Services\OcorrenciaService;
use Ems\Exception\CustomException;
use Ems\Services\VinculoService;
use Ems\Util\ExpectedException;
use Util\Environment;
use Ems\Util\Time;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\{ Error, Message };

class MarcacaoController {
    /** @var MarcacaoService */
    protected $marcacaoService;

    public function __construct(
        Environment $environment,
        MarcacaoService $marcacaoService,
        OcorrenciaService $ocorrenciaService,
        VinculoService $vinculoService
    ) {
        $this->environment          = $environment;
        $this->marcacaoService      = $marcacaoService;
        $this->ocorrenciaService    = $ocorrenciaService;
        $this->vinculoService       = $vinculoService;
    }

    public function getVinculosGestor(Request\Row $request, Response $response)
    {
        try {
            $row = $request->getRow();

            $nrEstrutura   = (int) $row['NRESTRUTURAM'];

            $nrPeriodo     = isset($row['NRPERIODOAPURACAO']) ?
                (int) $row['NRPERIODOAPURACAO'] :
                -1;

            $nrTipoVinculo = (int) $row['NRTIPOVINCULOM'];

            $vinculos = $this->vinculoService->getVinculosGestor($nrEstrutura, $nrPeriodo, $nrTipoVinculo);

            $response->addDataSet(new DataSet('getVinculosGestor', $vinculos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function setMarcacao(
        $nrvinculom,
        // $nrorg,
        $cdoperador,
        $dslatitude,
        $dslongitude,
        $marcacaoAlmoco = false,
        $photoContent = null
    ) {
        try {
            $nrorg = $this->environment->getNrorg();

            $nrmarcacaoponto = $this->marcacaoService->setMarcacao(
                $nrvinculom,
                $nrorg,
                $cdoperador,
                $dslatitude,
                $dslongitude,
                $photoContent
            );

            if ($marcacaoAlmoco) {
                $this->ocorrenciaService->setOcorrenciaAlmoco($nrvinculom, $nrorg, $nrmarcacaoponto, $cdoperador);
            }

            return true;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function registraMarcacaoIntervalo(Request\Row $request, Response $response)
    {
        try {
            $row = $request->getRow();

            $success = $this->setMarcacao(
                $row['NRVINCULOM'],
                $row['CDOPERADOR'],
                $row['DSLATITUDE'],
                $row['DSLONGITUDE'],
                false,
                $row['DSPATH']
            );
            $response->addDataSet(new DataSet('setMarcacaoIntervalo', array('success' => $success)));
        } catch (\Exception $e) {
            $variables = null;
            if ($e instanceof CustomException) {
                $variables = $e->getVariables();
            }
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR, null, $variables));
        }
    }

    public function registraMarcacaoAlmoco(Request\Row $request, Response $response)
    {
        try {
            $row = $request->getRow();

            $success = $this->setMarcacao(
                $row['NRVINCULOM'],
                $row['CDOPERADOR'],
                $row['DSLATITUDE'],
                $row['DSLONGITUDE'],
                true,
                $row['DSPATH']
            );
            $response->addDataSet(new DataSet('setMarcacaoAlmoco', array('success' => $success)));
        } catch (\Exception $e) {
            $variables = null;
            if ($e instanceof CustomException) {
                $variables = $e->getVariables();
            }
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR, null, $variables));
        }
    }

    public function validaMarcacaoDistancia(Request\Row $request, Response $response)
    {
        try {
            $row = $request->getRow();

            $validacoes = $this->marcacaoService->getValidacaoMarcacaoDistancia(
                $row['NRVINCULOM'],
                $this->environment->getNrorg(),
                $row['DSLATITUDE'],
                $row['DSLONGITUDE']
            );
            $response->addDataSet(new DataSet('validacoesMarcacao', array('validacoesMarcacao' => $validacoes)));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    public function getParesMarcacoesDia(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $nrVinculo = (int) $row['NRVINCULOM'];
            $dia       = $row['DIA'];

            $paresMarcacoes = $this->marcacaoService->
                getParesMarcacoesDia($nrVinculo, $dia);

            $response->addDataSet(new DataSet('getParesMarcacoesDia', $paresMarcacoes));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTraceAsString(), $e));
        }
    }

    public function removeParMarcacao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $nrVinculo         = (int) $row['NRVINCULOM'];
            $nrMarcacaoEntrada = (int) $row['NRMARCACAOENTRADA'];
            $nrMarcacaoSaida   = (int) $row['NRMARCACAOSAIDA'];

            $marcacoesRemovidas = $this->marcacaoService->
                removeParMarcacao($nrVinculo, $nrMarcacaoEntrada, $nrMarcacaoSaida);

            $dataSet = [
                'marcacoesRemovidas' => $marcacoesRemovidas
            ];

            $response->addDataSet(new DataSet('removeParMarcacao', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTraceAsString(), $e));
        }
    }
    
    public function removeParesMarcacao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $nrVinculo          = (int)     $row['NRVINCULOM'];
            $hrMarcacaoEntrada  = (string)  $row['HRENTRADA'];
            $hrMarcacaoSaida    = (string)  $row['HRSAIDA'];
            $date               = (array)   $row['DATE'];

            $marcacoesRemovidas = $this->marcacaoService->
                removeParesMarcacao($nrVinculo, $hrMarcacaoEntrada, $hrMarcacaoSaida, $date);

            $response->addDataSet(new DataSet('marcacoesRemovidas', $marcacoesRemovidas));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTraceAsString(), $e));
        }
    }

    public function removeMarcacao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $nrVinculo          = (int)     $row['NRVINCULOM'];
            $nrMarcacaoponto    = (string)  $row['NRMARCACAOPONTO'];

            $marcacoesRemovidas = $this->marcacaoService->
                removeMarcacao($nrVinculo, $nrMarcacaoponto);

            $response->addDataSet(new DataSet('marcacoesRemovidas', array($marcacoesRemovidas)));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTraceAsString(), $e));
        }
    }

    public function editParMarcacao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $nrVinculo         = (int) $row['NRVINCULOM'];
            $nrMarcacaoEntrada = (int) $row['NRMARCACAOENTRADA'];
            $nrMarcacaoSaida   = (int) $row['NRMARCACAOSAIDA'];
            $dia               = (string) $row['DIA'];
            $hrEntrada         = (string) $row['HRENTRADA'];
            $hrSaida           = (string) $row['HRSAIDA'];
            $idDiaInicio       = (string) $row['DIA_INICIO'];
            $idDiaFim          = (string) $row['DIA_FIM'];

            $nrMarcacoes = $this->marcacaoService->editParMarcacao(
                $nrVinculo,
                $nrMarcacaoEntrada,
                $nrMarcacaoSaida,
                $dia,
                $hrEntrada,
                $hrSaida,
                $idDiaInicio,
                $idDiaFim
            );

            $dataSet = [
                'nrMarcacoes' => $nrMarcacoes
            ];

            $response->addDataSet(new DataSet('editParMarcacao', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTraceAsString(), $e));
        }
    }
    
    public function editParesMarcacao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $nrVinculo          = (int)     $row['NRVINCULOM'];
            $hrMarcacaoEntrada  = (string)  $row['HRENTRADA'];
            $hrMarcacaoSaida    = (string)  $row['HRSAIDA'];
            $dia                = (array)   $row['DATE'];
            $novaHrEntrada      = (string)  $row['NOVAHRENTRADA'];
            $novaHrSaida        = (string)  $row['NOVAHRSAIDA'];

            $nrMarcacoes = $this->marcacaoService->editParesMarcacao(
                $nrVinculo,
                $hrMarcacaoEntrada,
                $hrMarcacaoSaida,
                $novaHrEntrada,
                $novaHrSaida,
                $dia
            );

            $response->addDataSet(new DataSet('editParesMarcacao', $nrMarcacoes));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTraceAsString(), $e));
        }
    }

    public function saveParMarcacao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $nrVinculo         = (int) $row['NRVINCULOM'];
            $dia               = (string) $row['DIA'];
            $hrEntrada         = (string) $row['HRENTRADA'];
            $hrSaida           = (string) $row['HRSAIDA'];
            $idDiaInicio        = (string) $row['DIA_INICIO'];
            $idDiaFim        = (string) $row['DIA_FIM'];

            $nrMarcacoes = $this->marcacaoService->saveParMarcacao(
                $nrVinculo, $dia, $hrEntrada,
                $hrSaida, $idDiaInicio, $idDiaFim
            );

            $dataSet = [
                'nrMarcacoes' => $nrMarcacoes
            ];

            $response->addDataSet(new DataSet('saveParMarcacao', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTraceAsString(), $e));
        }
    }
    
    public function saveParesMarcacao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $nrVinculo      = (int)     $row['NRVINCULOM'];
            $dias           = (array)   $row['DIA'];
            $hrEntrada      = (string)  $row['HRENTRADA'];
            $hrSaida        = (string)  $row['HRSAIDA'];

            $idDiaInicio    = isset($row['DIA_INICIO']) ?
                $row['DIA_INICIO'] :
                'FC';

            $idDiaFim       = isset($row['DIA_FIM']) ?
                $row['DIA_FIM'] :
                'FC';

            $nrMarcacoes = $this->marcacaoService->saveParesMarcacao(
                $nrVinculo,
                $dias,
                $hrEntrada,
                $hrSaida,
                $idDiaInicio,
                $idDiaFim
            );

            $dataSet = [
                'nrMarcacoes' => $nrMarcacoes
            ];

            $response->addDataSet(new DataSet('saveParesMarcacao', $dataSet));
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTraceAsString(), $e));
        }
    }

    public function getMarcacoesFotos(
        Request\Row $request,
        Response $response
    ) {
        try {
            $row = $request->getRow();

            $nrVinculo = (int)    $row['NRVINCULOM'];
            $date      = (string) $row['DATE'];

            $filesData = $this->marcacaoService->getMarcacoesFotos(
                $nrVinculo, $date
            );

            $response->addDataSet(new DataSet('getMarcacoesFotos', $filesData));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTraceAsString(), $e));
        }
    }
    
    public function setEntradasSaidasFixos(
        Request\Row $request,
        Response $response
    ) {
        try {
            $row = $request->getRow();

            $nrVinculom     = (int)     $row['NRVINCULOM'];
            $cdOperador     = (int)     $row['CDOPERADOR'];
            $marcacoes      = (array)   $row['ROWS'];

            $alteracoesRealizadasComSucesso = $this->marcacaoService->alteraEntradasSaidasFixadosComProjecao($nrVinculom, $cdOperador, $marcacoes);

            $responseData = array('success' => $alteracoesRealizadasComSucesso);

            $response->addDataSet(new DataSet('setEntradasSaidasFixos', $responseData));
        } catch (ExpectedException $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTraceAsString(), $e));
        }
    }

    public function fixaEntradaSaida(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrVinculo         = (int)    $row['NRVINCULOM'];
            $date              = (string) $row['DATE'];
            $nrMarcacaoEntrada = (int)    $row['NRMARCACAOENTRADA'];
            $nrMarcacaoSaida   = (int)    $row['NRMARCACAOSAIDA'];

            $status = $this->marcacaoService->fixaEntradaSaida(
                $nrVinculo, $date,
                $nrMarcacaoEntrada, $nrMarcacaoSaida
            );

            $response->addDataSet(new DataSet('fixaEntradaSaida', $status));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getMarcacoesPontoDiaOpcoesFixarMarcacao(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrVinculo = (int)    $row['NRVINCULOM'];
            $date      = (string) $row['DATE'];

            $marcacoes = $this->marcacaoService->getMarcacoesPontoDiaOpcoesFixarMarcacao(
                $nrVinculo,
                $date
            );

            $response->addDataSet(new DataSet('getMarcacoesPontoDiaOpcoesFixarMarcacao', $marcacoes));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTraceAsString(), $e));
        }
    }

    public function reverterEntradaSaida(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrVinculo = (int)    $row['NRVINCULOM'];
            $date      = (string) $row['DATE'];

            $success = $this->marcacaoService->reverterEntradaSaida(
                $nrVinculo,
                $date
            );

            $dataset = [
                'success' => $success
            ];

            $response->addDataSet(new DataSet('reverterEntradaSaida', $dataset));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTraceAsString(), $e));
        }
    }

    public function getDetalheAcompanhamento(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrVinculo = (int)    $row['NRVINCULOM'];
            $dates     = (array)  $row['DATAAPURACAO'];

            $marcacoes = $this->marcacaoService->getDetalheAcompanhamento(
                $nrVinculo, $dates
            );

            $response->addDataSet(new DataSet('getDetalheAcompanhamento', $marcacoes));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTraceAsString(), $e));
        }
    }
}

