<?php

namespace Ems\Services;

use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;

use Ems\Services\MarcacaoService;
use Ems\Services\OcorrenciaService;
use Ems\Services\VinculoService;
use Ems\Repository\IntervaloRepository;

use Ems\Util\Utils;
use Util\Environment;

class IntervaloService {

    /** @var MarcacaoService */
    protected $marcacaoService;

    /** @var OcorrenciaService */
    protected $ocorrenciaService;

    /** @var VinculoService */
    protected $vinculoService;

    /** @var Connection */
    protected $connection;

    /** @var Environment */
    protected $environment;

    /** @var IntervaloRepository */
    protected $intervaloRepository;

    public function __construct(
        MarcacaoService $marcacaoService,
        OcorrenciaService $ocorrenciaService,
        EntityManager $entityManager,
        Environment $environment,
        VinculoService $vinculoService,
        IntervaloRepository $intervaloRepository
    ) {
        $this->marcacaoService      = $marcacaoService;
        $this->ocorrenciaService    = $ocorrenciaService;
        $this->connection           = $entityManager->getConnection();
        $this->environment          = $environment;
        $this->vinculoService       = $vinculoService;
        $this->intervaloRepository  = $intervaloRepository;
    }

    public function insertIntervalo(
        int    $nrVinculo,
        string $date,
        string $horarioBaliza,
        bool   $before,
        array  $ocorrenciaRow
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $status = [
            'success' => true,
            'message' => null
        ];

        $marcacoes = $this->marcacaoService->getMarcacoesDia($nrVinculo, $date);

        if (empty($marcacoes)) {
            $status['success'] = false;
            $status['message'] = 'Não é possível adicionar intervalos em dias sem marcação!';

            return $status;
        }

        $intervaloExists = $this->verifyIntervaloExists($before, $marcacoes);

        if ($intervaloExists) {
            $message = '';

            if ($before) {
                $message = 'O dia já possui intervalo antes da jornada!';
            } else {
                $message = 'O dia já possui intervalo após a jornada!';
            }

            $status['success'] = false;
            $status['message'] = $message;

            return $status;
        }

        $marcacao = null;

        if ($before) {
            $marcacao = $marcacoes[0];
        } else {
            $lastIndex = count($marcacoes) - 1;
            $marcacao = $marcacoes[$lastIndex];
        }

        $marcHorario = $marcacao['HORA_MINUTO'];
        $marcId      = $marcacao['IDDIAOCORRENCIA'];
        $nrMarcacao  = $marcacao['NRMARCACAOPONTO'];

        $balizaDateAndId = self::getBalizaDateAndId(
            $date, $horarioBaliza, $marcHorario,
            $marcId, $before
        );

        if ($balizaDateAndId === false) {
            $status['success'] = false;
            $status['message'] = 'Horário de intervalo inválido!';

            return $status;
        }

        $balizaDate = $balizaDateAndId['date'];
        $balizaId = $balizaDateAndId['id'];

        $this->connection->beginTransaction();

        try {
            $nrMarcacoesBaliza = $this->marcacaoService->createBaliza(
                $nrVinculo, $balizaDate, $horarioBaliza, $balizaId
            );

            $ocorrenciaNrMarcacao = $before ?
                $nrMarcacoesBaliza[1] :
                $nrMarcacao;

            $ocorrenciaRow['NRMARCACAOPONTO'     ] = $ocorrenciaNrMarcacao;
            $ocorrenciaRow['IDPROGOCORCONFIRMADA'] = NULL;
            $ocorrenciaRow['NRTIPOPROGOCORRENCIA'] = 17; // Abonar saída
            $ocorrenciaRow['NRVINCULOM'          ] = $nrVinculo;
            $ocorrenciaRow['DTINICIOPROGOCOR'    ] = $date;
            $ocorrenciaRow['DTFIMPROGOCOR'       ] = $date;
            $ocorrenciaRow['__is_new'            ] = true;
            $ocorrenciaRow['NRORG'               ] = $nrOrg;

            $this->ocorrenciaService->setOcorrenciaFreq($ocorrenciaRow);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }

        return $status;
    }

    public static function getBalizaDateAndId(
        string $date,
        string $horarioBaliza,
        string $marcHorario,
        string $marcId,
        bool   $before
    ): array {
        $balizaDateAndId = null;

        if ($before) {
            $balizaDateAndId = self::getBalizaDateAndIdBefore(
                $date,
                $horarioBaliza,
                $marcHorario,
                $marcId
            );
        } else {
            $balizaDateAndId = self::getBalizaDateAndIdAfter(
                $date,
                $horarioBaliza,
                $marcHorario,
                $marcId
            );
        }

        return $balizaDateAndId;
    }

    public static function getBalizaDateAndIdBefore(
        string $date,
        string $horarioBaliza,
        string $primeiraMarcHorario,
        string $primeiraMarcId
    ): array {
        $balizaDate = '';
        $balizaId = '';

        // "13:10" => 1310
        $balizaHorarioNumber = (int) str_replace(':', '', $horarioBaliza);
        $marcHorarioNumber = (int) str_replace(':', '', $primeiraMarcHorario);

        // Desculpe pelo if/else hell :(

        // Dado que se trata de um intervalo anterior à marcação,
        // se o horário da baliza é superior ao da marcação,
        // o intervalo provavelmente é do dia anterior.
        if ($balizaHorarioNumber > $marcHorarioNumber) {
            // Se a marcação é do dia corrente, o intervalo é do dia anterior
            if ($primeiraMarcId === 'FC' || $primeiraMarcId === 'DC') {
                $primeiraMarcId === 'FC' ?
                    'FS' :
                    'DS';

                $balizaDate = Utils::subDaysFromDate($date, 1);
            // Se a marcação é do dia seguinte, o intervalo é do dia corrente
            } elseif ($primeiraMarcId === 'FA' || $primeiraMarcId === 'DA') {
                $primeiraMarcId === 'FA' ?
                    'FC' :
                    'DC';

                $balizaDate = $date;
            // Não tem como o intervalo ter o horário superior ao de uma
            // marcação do dia anterior
            } else {
                return false;
            }
        } else {
            $balizaDate = $date;
            $balizaId = $primeiraMarcId;
        }

        $dateAndId = [
            'date' => $balizaDate,
            'id'   => $balizaId
        ];

        return $dateAndId;
    }

    public static function getBalizaDateAndIdAfter(
        string $date,
        string $horarioBaliza,
        string $marcHorario,
        string $marcId
    ): array {
        $balizaDate = '';
        $balizaId = '';

        // "13:10" => 1310
        $balizaHorarioNumber = (int) str_replace(':', '', $horarioBaliza);
        $marcHorarioNumber = (int) str_replace(':', '', $marcHorario);

        // Desculpe pelo if/else hell :(

        // Dado que se trata de um intervalo anterior à marcação,
        // se o horário da baliza é superior ao da marcação,
        // o intervalo provavelmente é do dia anterior.
        if ($balizaHorarioNumber > $marcHorarioNumber) {
            $balizaDate = $date;
            $balizaId = $marcId;
        } else {
            // Se a marcação é do dia corrente, o intervalo é do dia seguinte
            if ($marcId === 'FC' || $marcId === 'DC') {
                $marcId = $marcId === 'FC' ?
                    'FA' :
                    'DA';

                $balizaDate = Utils::addDaysToDate($date, 1);
            // Se a marcação é do dia anterior, o intervalo é do dia corrente
            } elseif ($marcId === 'FS' || $marcId === 'DS') {
                $marcId = $marcId === 'FS' ?
                    'FC' :
                    'DC';

                $balizaDate = $date;
            // Não tem como o intervalo ter o horário inferior ao de uma
            // marcação do dia seguinte
            } else {
                return false;
            }
        }

        $dateAndId = [
            'date' => $balizaDate,
            'id'   => $balizaId
        ];

        return $dateAndId;
    }

    /**
     * Verifica se já existe algum intervalo antes ou depois
     * da jornada no dia em questão.
     */
    public function verifyIntervaloExists(
        bool   $before = true,
        array  $marcacoes = null,
        int    $nrVinculo = -1,
        string $date = ''
    ): bool {
        if ($marcacoes === null) {
            $marcacoes = $this->marcacaoService->getMarcacoesDia(
                $nrVinculo, $date
            );
        }

        if (empty($marcacoes)) {
            return false;
        }

        $lastIndex = count($marcacoes) - 1;

        $index = $before ?
            0 :
            $lastIndex;

        $marcacao = $marcacoes[$index];
        $isIntervalo = $marcacao['IDREFERINTERVALO'] === 'S';

        return $isIntervalo;
    }

    public function deleteIntervalo(
        int    $nrVinculo,
        string $date,
        string $hrInicio,
        string $hrFim,
        bool   $before
    ): array {
        $status = [
            'success' => true,
            'message' => false
        ];

        $nrOrg = (int) $this->environment->getNrOrg();

        $horarioBaliza = $before ?
            $hrInicio :
            $hrFim;

        $params = [
            'P_NRORG'       => $nrOrg,
            'P_NRVINCULO'   => $nrVinculo,
            'P_DATE'        => $date,
            'P_TIME'        => $horarioBaliza,
        ];

        $this->connection->beginTransaction();

        try {
            $horarioOcorrencia = $before ?
                $hrFim :
                $hrInicio;

            $this->ocorrenciaService->removeOcorrenciaByDateTime(
                $nrVinculo, $date, $horarioOcorrencia
            );

            $qntDeletedRows = $this->intervaloRepository->deleteIntervalo($params);

            if ($qntDeletedRows === 0) {
                $message = 'Não há nenhum intervalo no horário informado!';

                $status['success'] = false;
                $status['message'] = $message;

                return $status;
            }

            $this->connection->commit();

            return $status;
        } catch (\Exception $e) {
            $this->connection->rollBack();
            throw $e;
        }
    }
}